#include "stdafx.h"
#include <math.h>
#include "AccessObjects.h"
#include "ControlStellaB3.h"
#include "UnitConv.h"


CControlStellaB3::CControlStellaB3 ()
{
	// ⺻  
	_wheelRadius = 0.075f;
	_axleLength  = 0.289f;
	_encoderPPR  = 1024;
	_gearRatio   = 14;

	_leftEncoderPrev = 0;
	_rightEncoderPrev = 0;

	_x = _y = _theta = 0.;


	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ߿ !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	// κ  ٸ ŸӾƿ    COM Ʈ,  ӵ 
	// ⼭ Ѵ.
	int readTimeout = 100;
	_obj = new CAccessObjects ("\\\\.\\COM3", CBR_115200, readTimeout);
	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	if (_obj->IsConnected ()) {
		double value = 0.;
		if (_obj->ReadObject ("wheel_radius",	value)) _wheelRadius = (float)value;
		if (_obj->ReadObject ("axle_length",	value)) _axleLength  = (float)value;
		// ¿  ڴ PPR   1üο о´.
		if (_obj->ReadObject ("encoder_ppr1",	value)) _encoderPPR  = (short)value;	
		if (_obj->ReadObject ("gear_ratio",		value)) _gearRatio   = (float)value;

		if (_obj->ReadObject ("position1",		value)) _leftEncoderPrev   = (long)value;
		if (_obj->ReadObject ("position2",		value)) _rightEncoderPrev  = (long)value;
	}
	else {
		TRACE ("Serial port   ϴ.\n");
	}
}

CControlStellaB3::~CControlStellaB3 ()
{
	delete _obj;
}

bool CControlStellaB3::Drive (double linearVel, double angularVel)
{
	if (!_obj) return false;
	if (!_obj->IsConnected ()) return false;

	// ¿     ȸӵ RPM  Ѵ.
	float lvel = (float)(RPS2RPM*_gearRatio/_wheelRadius*(linearVel - _axleLength/2.*angularVel));
	float rvel = (float)(RPS2RPM*_gearRatio/_wheelRadius*(linearVel + _axleLength/2.*angularVel));

	//  ⿡ ӵ  .
	if (_obj->WriteObject ("velocity_command1",		(double)lvel));
	else TRACE ("  ӵ   ϴ.\n");

	if (_obj->WriteObject ("velocity_command2",		(double)rvel));
	else TRACE ("  ӵ   ϴ.\n");
	
	double value = 0.;
	long leftEncoder  = 0;
	long rightEncoder = 0;

	//  ⿡ ڴ  о´.
	if (_obj->ReadObject ("position1",		value)) leftEncoder  = (long)value;
	else TRACE ("  ڴ о  ϴ.\n");

	if (_obj->ReadObject ("position2",		value)) rightEncoder = (long)value;
	else TRACE ("  ڴ о  ϴ.\n");

	// ¿  ڴ  󸶳 ߴ 뷮 Ѵ.
	long dl = (leftEncoder  - _leftEncoderPrev);
	long dr = (rightEncoder - _rightEncoderPrev);

	_leftEncoderPrev  = leftEncoder;
	_rightEncoderPrev = rightEncoder;

	DeadReckoning (dl, dr);

	return true;
}

void CControlStellaB3::DeadReckoning (long dl, long dr)
{
	// ¿  ڴ ȭκ κ ġ(_x, _y, _theta)
	// ϱ  ⱸ Ѵ. ̸ Dead-reckoning ̶ Ѵ.

	double r = 2*M_PI*_wheelRadius/_encoderPPR/_gearRatio;
	double dlm = r*dl;
	double drm = r*dr;

	double v = (dlm + drm)/2.;
	double w = (drm - dlm)/_axleLength;

	if (-0.001 > w || w > 0.001) {
		_x += v/w*(sin(_theta + w) - sin(_theta));
		_y -= v/w*(cos(_theta + w) - cos(_theta));
	}
	else {
		_x += v*cos(_theta);
		_y += v*sin(_theta);
	}
	_theta += w;
}

